package it.inaf.oats.ia2.tapschemamanager;

import it.inaf.oats.ia2.tapschemamodel.TapSchema;
import it.inaf.oats.ia2.tapschemamodel.Credentials;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
@WebServlet(name = "TSMServlet", urlPatterns = {"/" + TSMPages.CREDENTIALS, "/" + TSMPages.SELECT_SCHEMA, "/" + TSMPages.EDIT_SCHEMA})
public class TSMServlet extends HttpServlet {

    @EJB
    private ApplicationManagerLocal applicationManager;

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String path = request.getRequestURI().substring(request.getContextPath().length());

        if (!applicationManager.isLoggedIn() && !path.startsWith("/" + TSMPages.CREDENTIALS)) {
            response.sendRedirect(TSMPages.CREDENTIALS);
        } else {
            if (path.startsWith("/" + TSMPages.CREDENTIALS)) {
                applicationManager.setCurrentPage(TSMPages.CREDENTIALS);
                showCredentialForm(request, response, new InsertCredentialsModel(applicationManager.getCredentials()));
            } else if (path.startsWith("/" + TSMPages.SELECT_SCHEMA)) {
                applicationManager.setCurrentPage(TSMPages.SELECT_SCHEMA);
                request.setAttribute("model", applicationManager.getSelectTapSchema());
                request.getRequestDispatcher("/" + TSMPages.SELECT_SCHEMA + ".jsp").forward(request, response);
            } else if (path.startsWith("/" + TSMPages.EDIT_SCHEMA)) {
                applicationManager.setCurrentPage(TSMPages.EDIT_SCHEMA);
                request.setAttribute("tapSchemaName", applicationManager.getEditTapSchemaPageState().getTapSchema().getName());
                request.getRequestDispatcher("/" + TSMPages.EDIT_SCHEMA + ".jsp").forward(request, response);
            }
        }
    }

    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String path = request.getRequestURI().substring(request.getContextPath().length());

        if (path.startsWith("/" + TSMPages.CREDENTIALS)) {
            processInsertCredentials(request, response);
        } else if (path.startsWith("/" + TSMPages.SELECT_SCHEMA)) {
            Connection connection = null;
            try {
                connection = applicationManager.getCredentials().getConnection();

                TapSchema tapSchema;
                if (request.getParameter("tapschemaRadioGroup").equals("create")) {
                    String tapSchemaName = request.getParameter("tapschemaName");
                    ArrayList<String> selectedSchemas = new ArrayList<String>();
                    String[] selectedSchemasParam = request.getParameterValues("selectedSchemas");
                    if (selectedSchemasParam != null) {
                        selectedSchemas.addAll(Arrays.asList(selectedSchemasParam));
                    }
                    tapSchema = new TapSchema(connection, tapSchemaName, selectedSchemas);
                } else {
                    String tapSchemaName = request.getParameter("tapschemaDropDownChoice");
                    tapSchema = new TapSchema(connection, tapSchemaName);
                }

                applicationManager.setEditTapSchemaPageState(new EditTapSchemaPageState(tapSchema));
                response.sendRedirect(TSMPages.EDIT_SCHEMA);
            } catch (SQLException e) {
                e.printStackTrace(System.err);
            } finally {
                try {
                    if (connection != null) {
                        connection.close();
                    }
                } catch (SQLException e) {
                    e.printStackTrace(System.err);
                }
            }
        }
    }

    private void showCredentialForm(HttpServletRequest request, HttpServletResponse response, InsertCredentialsModel insertCredentialsModel)
            throws ServletException, IOException {
        request.setAttribute("insertCredentials", insertCredentialsModel);
        request.getRequestDispatcher("/" + TSMPages.CREDENTIALS + ".jsp").forward(request, response);
    }

    private void processInsertCredentials(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String hostname = request.getParameter("hostname");
        String portStr = request.getParameter("port");
        String username = request.getParameter("username");
        String password = request.getParameter("password");

        Credentials credentials = applicationManager.getCredentials();
        InsertCredentialsModel insertCredentialsModel = new InsertCredentialsModel(credentials);

        if (hostname != null || portStr != null || username != null) {
            if (hostname == null || hostname.equals("")) {
                insertCredentialsModel.addError("hostname can't be empty");
            } else {
                credentials.setHostname(hostname);
            }

            if (portStr == null || portStr.equals("")) {
                insertCredentialsModel.addError("port can't be empty");
            }
            int port;
            try {
                port = Integer.parseInt(portStr);
                credentials.setPort(port);
            } catch (NumberFormatException e) {
                insertCredentialsModel.addError("invalid port value");
            }

            if (username == null || username.equals("")) {
                insertCredentialsModel.addError("username can't be empty!");
            } else {
                credentials.setUsername(username);
            }

            if (password == null) {
                password = "";
            }
            credentials.setPassword(password);
        }

        if (insertCredentialsModel.getFormErrors().isEmpty()) {
            try {
                SelectTapSchemaModel selectTapSchema = new SelectTapSchemaModel(credentials.getConnection());
                applicationManager.setSelectTapSchemaModel(selectTapSchema);
                request.getSession().setAttribute("applicationManager", applicationManager);
                response.sendRedirect(TSMPages.SELECT_SCHEMA);
            } catch (SQLException e) {
                insertCredentialsModel.addError("Connection error: " + e.getMessage());
            }
        }

        if (!applicationManager.isLoggedIn()) {
            showCredentialForm(request, response, insertCredentialsModel);
        }
    }
}
