package it.inaf.oats.ia2.tapschemamodel;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author pietro
 */
public class Schema extends DbEntity implements IDbEntitiesContainer, Serializable {

    public static final String UTYPE = "utype";
    public static final String DESCRIPTION = "description";

    private final String name;
    private final String tapSchemaName;

//    private final EditableValue<String> utype;
//    private final EditableValue<String> description;
    //private String utype;
    //private String description;
    private final Map<String, Table> tablesMap;
    //private final Map<String, Key> keysMap;
    private final List<Key> keysInfo;

    private String selectedTable;

    public Schema(Connection connection, String schemaName, String tapSchemaName, boolean toCreate) throws SQLException {
        super(UTYPE, DESCRIPTION);

        this.name = schemaName;
        this.tapSchemaName = tapSchemaName;
        setStatus(Status.NOT_SELECTED);

        tablesMap = new HashMap<String, Table>();
        for (String tableName : SchemaDL.getAllTablesNames(connection, schemaName)) {
            tablesMap.put(tableName, null);
        }

        keysInfo = new ArrayList<Key>();
        (new ResultSetReader("SELECT k.TABLE_NAME, k.COLUMN_NAME, "
                + "k.REFERENCED_TABLE_NAME, k.REFERENCED_COLUMN_NAME "
                + "FROM information_schema.TABLE_CONSTRAINTS i "
                + "LEFT JOIN information_schema.KEY_COLUMN_USAGE k "
                + "ON i.CONSTRAINT_NAME = k.CONSTRAINT_NAME "
                + "WHERE i.CONSTRAINT_TYPE = 'FOREIGN KEY' "
                + "AND i.TABLE_SCHEMA = '" + schemaName + "' "
                + "AND k.TABLE_SCHEMA = '" + schemaName + "'") {

                    @Override
                    public void manipulateItem(ResultSet resultSet) throws SQLException {
                        addKey(
                                resultSet.getString("k.TABLE_NAME"),
                                resultSet.getString("k.REFERENCED_TABLE_NAME"),
                                resultSet.getString("k.COLUMN_NAME"),
                                resultSet.getString("k.REFERENCED_COLUMN_NAME")
                        );
                    }
                }).read(connection);

        //keysMap = new HashMap<String, Key>();
        //SchemaDL.setKeysMap(connection, tapSchemaName, schemaName, keysMap);
        if (!toCreate) {
            SchemaDL.loadSavedTables(connection, tapSchemaName, (this), toCreate);
        }

//        utype = new EditableValue<String>();
//        description = new EditableValue<String>();
    }

    private void addKey(String fromTable, String targetTable, String fromColumn, String targetColumn) {
        fromTable = getName() + "." + fromTable;
        targetTable = getName() + "." + targetTable;

        Key key = null;
        for (Key k : keysInfo) {
            if (k.getFromTable().equals(fromTable) && k.getTargetTable().equals(targetTable)) {
                key = k;
            }
        }

        if (key == null) {
            key = new Key(fromTable, targetTable);
            keysInfo.add(key);
        }

        key.addKeyColumn(fromColumn, targetColumn);
    }

    @Override
    public void loadEntities(Connection connection, List<String> selectedTables) throws SQLException {
        for (String tableName : selectedTables) {
            Table table = SchemaDL.loadTable(connection, tapSchemaName, this, tableName);
            table.setStatus(Status.SELECTED_TO_SAVE);
            addEntity(tableName, table);
        }
        select(selectedTables.get(selectedTables.size() - 1));
    }

    public Table getTable(String tableName) {
        return tablesMap.get(tableName);
    }

    @Override
    public String getName() {
        return name;
    }

//    public EditableValue<String> getUtypeEditableValue() {
//        return utype;
//    }
//
//    public String getUtype() {
//        return utype.getValue();
//    }
//
//    public void setUtype(String utype) {
//        this.utype.setValue(utype);
//    }
//    public EditableValue<String> getDescriptionEditableValue() {
//        return description;
//    }
//    public String getDescription() {
//        return this.description.getValue();
//    }
//
//    public void setDescription(String description) {
//        this.description.setValue(description);
//    }
    @Override
    public void addEntity(String tableName, DbEntity table) {
        tablesMap.put(tableName, (Table) table);
    }

//    public Collection<String> getTablesNames() {
//        return tablesMap.keySet();
//    }
    @Override
    public ArrayList<DbEntity> getAllEntities() {
        ArrayList<DbEntity> list = new ArrayList<DbEntity>();
        for (Table table : tablesMap.values()) {
            if (table != null) {
                list.add(table);
            }
        }
        return list;
    }

    @Override
    public ArrayList<String> getAddables() {
        ArrayList<String> addableTables = new ArrayList<String>();

        for (String tableName : tablesMap.keySet()) {
            if (tablesMap.get(tableName) == null) {
                addableTables.add(tableName);
            }
        }

        return addableTables;
    }

    @Override
    public void removeEntity(String tableName) {
        Table table = getTable(tableName);
        if (table.getStatus() == Status.SELECTED_SAVED) {
            table.setStatus(Status.TO_REMOVE);
        }
        if (table.getStatus() == Status.SELECTED_TO_SAVE) {
            tablesMap.put(tableName, null);
            ArrayList<DbEntity> entities = getAllEntities();
            if (!entities.isEmpty()) {
                select(entities.get(0).getName());
            }
        }
    }

    public boolean isForeignKey(String tableName, String columnName) {
        for (Key key : keysInfo) {
            if (key.getFromTable().equals(tableName) && key.hasFromKeyColumn(columnName)) {
                return true;
            }
        }
        return false;
    }

//    @Override
//    public String getOriginalValue(String key) {
//        if (key.equals(UTYPE)) {
//            return this.getUtype();
//        }
//        if (key.equals(DESCRIPTION)) {
//            return this.getDescription();
//        }
//        return null;
//    }
    @Override
    public DbEntity getSelected() {
        return tablesMap.get(selectedTable);
    }

    @Override
    public void select(String name) {
        selectedTable = name;

        // select first column
        Table table = tablesMap.get(selectedTable);
        ArrayList<DbEntity> columns = table.getAllEntities();
        if (!columns.isEmpty()) {
            table.select(columns.get(0).getName());
        }
    }

    @Override
    public void insert(Connection connection) throws SQLException {
        PreparedStatement statement = null;
        try {
            statement = connection.prepareStatement("INSERT INTO `schemas` (schema_name, utype, description, schemaID) "
                    + "VALUES(?, ?, ? , null);");
            statement.setString(1, getName());
            statement.setString(2, getValue(UTYPE));
            statement.setString(3, getValue(DESCRIPTION));
            statement.executeUpdate();
        } catch (SQLException e) {
            throw e;
        } finally {
            if (statement != null) {
                statement.close();
            }
        }
    }

    @Override
    public void update(Connection connection) throws SQLException {
        PreparedStatement statement = null;
        try {
            statement = connection.prepareStatement("UPDATE `schemas` "
                    + "SET utype = ?, description = ? WHERE schema_name = ?");
            statement.setString(1, getValue(UTYPE));
            statement.setString(2, getValue(DESCRIPTION));
            statement.setString(3, getName());
            statement.executeUpdate();
        } catch (SQLException e) {
            throw e;
        } finally {
            if (statement != null) {
                statement.close();
            }
        }
    }

    public List<Key> getKeys() {
        return keysInfo;
    }

    @Override
    public void update() {
        setUpdated();
        for (DbEntity tableEntity : getAllEntities()) {
            Table table = (Table) tableEntity;
            if (table.getStatus() == Status.TO_REMOVE) {
                tablesMap.put(table.getName(), null);
            } else {
                table.update();
            }
        }
    }
}
