package it.inaf.oats.ia2.tapschemamodel;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class SchemaDL {

    public static ArrayList<String> getAllTablesNames(Connection connection, String schemaName) throws SQLException {
        final ArrayList<String> allTables = new ArrayList<String>();

        (new ResultSetReader("SHOW TABLES FROM " + schemaName + ";") {

            @Override
            public void manipulateItem(ResultSet resultSet) throws SQLException {
                allTables.add(resultSet.getString(1));
            }
        }).read(connection);

        return allTables;
    }

    public static void setKeysMap(final Connection connection, final String tapSchemaName, final String schemaName, final Map<String, Key> keysMap) throws SQLException {

        final HashMap<String, Key> keys = new HashMap<String, Key>();

        (new ResultSetReader("SELECT k.CONSTRAINT_NAME, k.TABLE_NAME, k.COLUMN_NAME, "
                + "k.REFERENCED_TABLE_NAME, k.REFERENCED_COLUMN_NAME "
                + "FROM information_schema.TABLE_CONSTRAINTS i "
                + "LEFT JOIN information_schema.KEY_COLUMN_USAGE k "
                + "ON i.CONSTRAINT_NAME = k.CONSTRAINT_NAME "
                + "WHERE i.CONSTRAINT_TYPE = 'FOREIGN KEY' "
                + "AND i.TABLE_SCHEMA = '" + schemaName + "' "
                + "AND k.TABLE_SCHEMA = '" + schemaName + "';") {

                    @Override
                    public void manipulateItem(ResultSet resultSet) throws SQLException {
                        String constraintName = resultSet.getString("k.CONSTRAINT_NAME");
                        String fromTable = schemaName + "." + resultSet.getString("k.TABLE_NAME");
                        String targetTable = schemaName + "." + resultSet.getString("k.REFERENCED_TABLE_NAME");
                        String fromColumn = resultSet.getString("k.COLUMN_NAME");
                        String targetColumn = resultSet.getString("k.REFERENCED_COLUMN_NAME");

                        Key key = keys.get(constraintName);
                        if (key == null) {
                            String keyId = "" + keys.size() + 1;
                            key = new Key(fromTable, targetTable);
                            keys.put(constraintName, key);
                            keysMap.put(keyId, key);
                        }
                        key.addKeyColumn(fromColumn, targetColumn);
                    }
                }).read(connection);
    }

    public static Table loadTable(final Connection connection, final String tapSchemaName, final Schema schema, final String tableName) throws SQLException {
        Table table = new Table(connection, tapSchemaName, schema, tableName, true);
        for (DbEntity column : table.getAllEntities()) {
            // Select all table columns by default
            column.setStatus(Status.SELECTED_TO_SAVE);
        }
        return table;
    }

    public static void loadSavedTables(final Connection connection, final String tapSchemaName, final Schema schema, final boolean toCreate) throws SQLException {

        (new ResultSetReader("SELECT * FROM " + tapSchemaName + ".tables "
                + "WHERE schema_name='" + schema.getName() + "';") {

                    @Override
                    public void manipulateItem(ResultSet resultSet) throws SQLException {

                        String tableName = resultSet.getString("table_name");
                        // Removing schema prefix from table name
                        if (tableName.startsWith(schema.getName() + ".")) {
                            tableName = tableName.substring(schema.getName().length() + 1);
                        }

                        Table table = new Table(connection, tapSchemaName, schema, tableName, toCreate);
                        
                        table.setValue(Table.UTYPE, resultSet.getString("utype"));
                        table.setValue(Table.DESCRIPTION, resultSet.getString("description"));
                        
                        table.setStatus(Status.SELECTED_SAVED);
                        schema.addEntity(tableName, table);
                    }
                }).read(connection);
    }

}
