package it.inaf.oats.ia2.tapschemamodel;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class TableDL {

    private static boolean equalsOneOf(String string, String... values) {
        for (String value : values) {
            if (string.equals(value)) {
                return true;
            }
        }
        return false;
    }

    // updates variables of current column structure
    public static void loadAllColumns(Connection connection, final String tapSchemaName, final String schemaName, final Table table) throws SQLException {

        (new ResultSetReader("SHOW COLUMNS FROM " + schemaName + "." + table.getName() + ";") {

            @Override
            public void manipulateItem(ResultSet resultSet) throws SQLException {

                Column column = new Column();
                column.setTableName(table.getFullName());
                column.setStatus(Status.NOT_SELECTED);

                // Key
                String key = resultSet.getString("Key");
                column.setPrincipal(key.equals("PRI"));
                column.setIsPrimaryKey(key.equals("PRI"));
                column.setIndexed(equalsOneOf(key, "PRI", "UNI", "MUL"));

                // Datatype and Size
                String type = resultSet.getString("Type").toLowerCase();

                int size = 0;
                String datatype;
                if (type.startsWith("int")) {
                    datatype = "adql:INTEGER";
                } else if (type.startsWith("smallint")) {
                    datatype = "adql:SMALLINT";
                } else if (type.startsWith("bigint")) {
                    datatype = "adql:BIGINT";
                } else if (type.startsWith("float")) {
                    datatype = "adql:REAL";
                } else if (type.startsWith("char")) {
                    int beginIndex = type.indexOf('(');
                    int endIndex = type.indexOf(')');
                    size = Integer.parseInt(type.substring(beginIndex + 1, endIndex));
                    datatype = "adql:CHAR";
                } else if (type.startsWith("varchar")) {
                    int beginIndex = type.indexOf('(');
                    int endIndex = type.indexOf(')');
                    size = Integer.parseInt(type.substring(beginIndex + 1, endIndex));
                    datatype = "adql:VARCHAR";
                } else if (type.contains("timestamp")) {
                    datatype = "adql:TIMESTAMP";
                } else {
                    datatype = "adql:" + type.toUpperCase();
                }
                column.setDatatype(datatype);
                column.setSize(size);

                // Column name
                String columnName = resultSet.getString("Field");
                column.setName(columnName);

                // STD
                boolean std = false;
                if ((schemaName.equals(tapSchemaName))
                        && !equalsOneOf(columnName, "schemaID", "tableID", "columnID", "keyID", "key_columnID")) {
                    std = true;
                }
                column.setStd(std);

                column.setIsForeignKey(table.isForeignKey(columnName));
                table.addEntity(columnName, column);
            }
        }).read(connection);
    }

    public static void loadSavedColumns(Connection connection, final String tapSchemaName, final String schemaName, final Table table) throws SQLException {

        (new ResultSetReader("SELECT * FROM " + tapSchemaName + ".columns "
                + "WHERE table_name='" + schemaName + "." + table.getName() + "';") {

                    @Override
                    public void manipulateItem(ResultSet columnsResultSet) throws SQLException {

                        Column column = table.getColumn(columnsResultSet.getString("column_name"));
                        column.setStatus(Status.SELECTED_SAVED);

                        // updates variables of current column structure
                        column.setTableName(columnsResultSet.getString("table_name"));
                        column.setValue(Column.UTYPE, columnsResultSet.getString("utype"));
                        column.setValue(Column.UCD, columnsResultSet.getString("ucd"));
                        column.setValue(Column.UNIT, columnsResultSet.getString("unit"));
                        column.setValue(Column.DESCRIPTION, columnsResultSet.getString("description"));
                        column.setDatatype(columnsResultSet.getString("datatype"));
                        column.setSize(columnsResultSet.getInt("size"));
                        column.setPrincipal(columnsResultSet.getInt("principal") == 1);
                        column.setIndexed(columnsResultSet.getInt("indexed") == 1);
                        column.setStd(columnsResultSet.getInt("std") == 1);

                        //column.checked = true;
                        //search if foreign key
//                        statement = connection.createStatement();
//                        queryString = "SELECT * FROM " + tapschemaName + ".keys "
//                        + "INNER JOIN " + tapschemaName + ".key_columns "
//                        + "USING (key_id) "
//                        + "WHERE " + tapschemaName + ".keys.from_table='" + column.getTableName() + "' "
//                        + "AND " + tapschemaName + ".key_columns.from_column='" + column.getColumnName() + "';";
//                        keysResultSet = statement.executeQuery(queryString);
//
//                        statement = connection.createStatement();
//                        queryString = "SELECT * FROM " + tapschemaName + ".keys "
//                        + "INNER JOIN " + tapschemaName + ".key_columns "
//                        + "USING (key_id) "
//                        + "WHERE " + tapschemaName + ".keys.target_table='" + column.getTableName() + "' "
//                        + "AND " + tapschemaName + ".key_columns.target_column='" + column.getColumnName() + "';";
//                        foreignKeysResultSet = statement.executeQuery(queryString);
//                        while (keysResultSet.next()) {
//                            while (foreignKeysResultSet.next()) {
//                                for (Key key : schema.keysMap.values()) {
//                                    for (KeyColumn key_column : schema.key_columnsMap.values()) {
//                                        if (key.from_table.equals(keysResultSet.getString("from_table"))
//                                        && key_column.from_column.equals(foreignKeysResultSet.getString("from_column"))
//                                        && keysResultSet.getString("key_id").equals(foreignKeysResultSet.getString("key_id"))) {
//                                            // updates variables of current key structure
//                                            key.key_id = keysResultSet.getString("key_id");
//                                            key.utype = keysResultSet.getString("utype");
//                                            key.description = keysResultSet.getString("description");
//
//                                            // updates variables of current key_column structure
//                                            key_column.key_id = foreignKeysResultSet.getString("key_id");
//                                        }
//                                    }
//                                }
//                            }
//                        }
                    }
                }).read(connection);
    }
}
