package it.inaf.oats.ia2.tapschemamodel;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class TapSchemaDL {

    public static List<String> getAllSchemasNames(Connection connection) throws SQLException {
        final List<String> allSchemas = new ArrayList<String>();
        (new ResultSetReader("SHOW DATABASES;") {

            @Override
            public void manipulateItem(ResultSet resultSet) throws SQLException {
                allSchemas.add(resultSet.getString(1));
            }

        }).read(connection);
        return allSchemas;
    }

    public static List<String> getAllTAPSchemasNames(Connection connection, final List<String> allSchemas) throws SQLException {

        class TAPSchemaChecker {

            boolean schemas;
            boolean tables;
            boolean columns;
            boolean keys;
            boolean key_columns;

            // if database has schemas, tables, columns, keys and key_columns
            // then it is a tap_schema database
            boolean isTAPSchema() {
                return schemas && tables && columns && keys && key_columns;
            }
        }

        final List<String> allTAPSchemas = new ArrayList<String>();

        for (String schemaName : allSchemas) {

            final TAPSchemaChecker tapSchemaChecker = new TAPSchemaChecker();

            (new ResultSetReader("SHOW TABLES FROM " + schemaName + ";") {

                @Override
                public void manipulateItem(ResultSet resultSet) throws SQLException {
                    // searches if the current database has schemas, tables, columns,
                    // keys, key_columns, that is a tap_schema database

                    String tableName = resultSet.getString(1);

                    if (tableName.equals("schemas")) {
                        tapSchemaChecker.schemas = true;
                    } else if (tableName.equals("tables")) {
                        tapSchemaChecker.tables = true;
                    } else if (tableName.equals("columns")) {
                        tapSchemaChecker.columns = true;
                    } else if (tableName.equals("keys")) {
                        tapSchemaChecker.keys = true;
                    } else if (tableName.equals("key_columns")) {
                        tapSchemaChecker.key_columns = true;
                    }
                }

            }).read(connection);

            if (tapSchemaChecker.isTAPSchema()) {
                allTAPSchemas.add(schemaName);
            }
        }

        return allTAPSchemas;
    }

    public static List<String> getExposedSchemas(Connection connection, final String tapSchemaName) throws SQLException {
        final List<String> exposedSchemas = new ArrayList<String>();

        (new ResultSetReader("SELECT schema_name FROM " + tapSchemaName + ".`schemas`") {

            @Override
            public void manipulateItem(ResultSet resultSet) throws SQLException {
                exposedSchemas.add(resultSet.getString(1));
            }
        }).read(connection);

        return exposedSchemas;
    }

    public static void loadSelectedSchemas(final Connection connection, TapSchema tapSchema, List<String> selectedSchemas) throws SQLException {
        for (String schemaName : selectedSchemas) {
            Schema schema = new Schema(connection, schemaName, tapSchema.getName(), tapSchema.toCreate());
            schema.setStatus(Status.SELECTED_TO_SAVE);
            tapSchema.addEntity(schemaName, schema);
        }
    }

    public static void loadSavedSchemas(final Connection connection, final TapSchema tapSchema) throws SQLException {

        (new ResultSetReader("SELECT * FROM " + tapSchema.getName() + ".`schemas`;") {

            @Override
            public void manipulateItem(ResultSet resultSet) throws SQLException {
                String schemaName = resultSet.getString("schema_name");

                Schema schema = new Schema(connection, schemaName, tapSchema.getName(), tapSchema.toCreate());
                schema.setStatus(Status.SELECTED_SAVED);

                schema.setValue(Schema.UTYPE, resultSet.getString("utype"));
                schema.setValue(Schema.DESCRIPTION, resultSet.getString("description"));
                // And schemaID ??

                tapSchema.addEntity(schemaName, schema);
            }
        }).read(connection);
    }

    public static void update(Credentials credentials, TapSchema tapSchema) {

    }
}
